@@ -1,5 +1,5 @@
 test_that("timestamp string parsing to offsets and deparsing of offsets to timestamps", {
-  # This test tests: global CFt* constants, CFtime(), CFparse(), CFtimestamp():
+  # This test tests: global CFt* constants, CFtime(), CFparse(), as_timestamp():
   # decomposing offsets into timestamp elements, generating timestamp strings,
   # parsing timestamp strings back into timestamp elements.
   for (c in CFt$calendars$name) {
@@ -8,7 +8,7 @@ test_that("timestamp string parsing to offsets and deparsing of offsets to times
       def <- paste(CFt$units$name[u], "since 1953-08-20")
       cf <- CFtime(def, c, offsets)
       time <- .offsets2time(cf@offsets, cf@datum)
-      ts <- CFtimestamp(cf, "timestamp")
+      ts <- as_timestamp(cf, "timestamp")
       cf2 <- CFtime(def, c)
       tp <- CFparse(cf2, ts)
       expect_equal(tp, time)
@@ -25,7 +25,7 @@ test_that("testing calendars with leap years", {
       def <- paste("days since", d)
       cf <- CFtime(def, c, c(1:2500, 36501:39000))
       time <- .offsets2time(cf@offsets, cf@datum)
-      ts <- CFtimestamp(cf, "timestamp")
+      ts <- as_timestamp(cf, "timestamp")
       cf2 <- CFtime(def, c)
       tp <- CFparse(cf2, ts)
       expect_equal(tp, time)
@@ -35,7 +35,7 @@ test_that("testing calendars with leap years", {
 
 test_that("Testing milli-second timestamp string parsing to offsets and deparsing
           of offsets to timestamps", {
-  # This test tests: global CFt* constants, CFtime(), CFparse(), CFtimestamp():
+  # This test tests: global CFt* constants, CFtime(), CFparse(), as_timestamp():
   # decomposing offsets into milli-second timestamp elements, generating timestamp strings,
   # parsing timestamp strings back into timestamp elements.
   for (c in CFt$calendars$name) {
@@ -44,7 +44,7 @@ test_that("Testing milli-second timestamp string parsing to offsets and deparsin
       def <- paste(CFt$units$name[u], "since 1953-08-20 07:34:12.2")
       cf <- CFtime(def, c, offsets)
       time <- .offsets2time(cf@offsets, cf@datum)
-      ts <- CFtimestamp(cf, "timestamp")
+      ts <- as_timestamp(cf, "timestamp")
       cf2 <- CFtime(def, c)
       tp <- CFparse(cf2, ts)
       expect_equal(tp[1:6], time[1:6])
@@ -57,7 +57,7 @@ test_that("Disallow parsing of timestamps on month and year datums", {
     for (u in 5:6) {
       def <- paste(CFt$units$name[u], "since 2020-01-01")
       cf <- CFtime(def, c, 0:23)
-      ts <- CFtimestamp(cf, "timestamp")
+      ts <- as_timestamp(cf, "timestamp")
       expect_error(CFparse(cf, ts))
     }
   }
